package com.your_domain.your_product.employee.gui;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.Image;
import java.awt.Point;
import java.awt.Toolkit;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.net.URL;
import java.text.DateFormat;
import java.util.ArrayList;
import java.util.Date;

import javax.swing.Box;
import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.ListSelectionModel;
import javax.swing.SwingUtilities;
import javax.swing.table.AbstractTableModel;
import javax.swing.table.DefaultTableCellRenderer;

import com.jdbaccess.da.DAFactory;
import com.jdbaccess.da.Result;
import com.jdbaccess.da.Transaction;
import com.jdbaccess.db.JDBaccess;
import com.jdbaccess.exception.ApplicationException;
import com.jdbaccess.transfer.Field;
import com.jdbaccess.transfer.TransferObject;
import com.your_domain.your_product.employee.session.EmployeeSession;
import com.your_domain.your_product.employee.session.EmployeeSessionImpl;
import com.your_domain.your_product.employee.transfer.Employee;
import com.your_domain.your_product.employee.transfer.EmployeeVO;
import com.your_domain.your_product.main.ExampleJDBAccess;

/**
 * GUI-layer: main frame for listing employees
 */
public class EmployeesFrame extends JFrame {
  static final long serialVersionUID = EmployeesFrame.class.hashCode();
  private static int PAGE_SIZE = 50;
  private ExampleJDBAccess jdbAccess;
  private Transaction transaction;  // main transaction
  private EmployeeSession empSession;
  private Result result;
  private JMenuItem aboutMenuItem;
  private JTable employeeTable = new JTable();
  private JButton prevPageButton = new JButton("<");
  private JButton nextPageButton = new JButton(">");
  private JButton okButton = new JButton("OK");
    
  public EmployeesFrame(ExampleJDBAccess jdbAccess) {
    super("JDBaccess-Example: Employees");
    this.jdbAccess = jdbAccess;
    initData();
    initGui();
    initListeners();    
    Dimension dim = getToolkit().getScreenSize();
    setSize(500, 250);
    setLocation((int) ((dim.width - getSize().getWidth()) / 2),
                (int) ((dim.height - getSize().getHeight()) / 2));
    setVisible(true);
  }

  public void refreshFromDB() {
    findAll();
    nextPage();
  }
  
  public void setDataAtSelectedRow(TransferObject transferObject) {
    EmployeeTableModel empTableModel = (EmployeeTableModel) employeeTable.getModel();
    if (empTableModel != null) {
      int row = employeeTable.getSelectedRow();
      empTableModel.setData(transferObject, row);
      empTableModel.fireTableDataChanged();
    }
  }

  public void refreshTable() {
    EmployeeTableModel empTableModel = (EmployeeTableModel) employeeTable.getModel();
    if (empTableModel != null) {
      empTableModel.fireTableDataChanged();
    }
  }
  
  private void initGui() {
    Image icon = JDBaccess.getIconImage();
    setIconImage(icon);
    JPanel panelCenter = createCenterPanel();
    JPanel panelSouth = createSouthPanel();
    getContentPane().add(panelCenter, BorderLayout.CENTER);
    getContentPane().add(panelSouth, BorderLayout.SOUTH);
    JMenuBar menuBar = new JMenuBar();
    JMenu menu = new JMenu("Help");
    menuBar.add(Box.createHorizontalGlue()); // to get it to the right end
    menuBar.add(menu);
    aboutMenuItem = new JMenuItem("About");
    menu.add(aboutMenuItem);
    setJMenuBar(menuBar);
  }

  private JPanel createCenterPanel() {
    JPanel panel_center = new JPanel(new BorderLayout());
    JScrollPane pane = new JScrollPane(employeeTable);
    panel_center.add(pane, BorderLayout.CENTER);
    return panel_center;
  }

  private JPanel createSouthPanel(){
    JPanel panel_south = new JPanel(new FlowLayout(FlowLayout.RIGHT));
    panel_south.add(prevPageButton);
    panel_south.add(nextPageButton);
    panel_south.add(okButton);
    return panel_south;
  }

  private void initData() {
    try {
      transaction = DAFactory.getTransaction();
      transaction.begin();
      empSession = new EmployeeSessionImpl(transaction);
      findAll();
      ArrayList fields = result.getFields();
      EmployeeTableModel empTableModel = new EmployeeTableModel(fields);
      employeeTable = new JTable(empTableModel);
      employeeTable.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
      DateRenderer dateRenderer = new DateRenderer();
      employeeTable.setDefaultRenderer(Date.class, dateRenderer);
      nextPage();
    } catch (ApplicationException e) {
      System.out.println(e.toString());
    }
  }

  private void findAll() {
    try {
      if (result != null)
        result.close();
      ArrayList orderById = new ArrayList();
      orderById.add("id");
      result = empSession.findEmployeeAll(orderById);
      result.setPageSize(PAGE_SIZE);
    } catch (ApplicationException e) {
      System.out.println(e.toString());
    }
  }
  
  private void openAboutDialog() {
    new AboutDialog(this);
  }
  
  private void initListeners() {
    aboutMenuItem.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        openAboutDialog();
      }
    });
    employeeTable.addMouseListener(new EmployeeMouseListener(this, employeeTable));
    nextPageButton.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        nextPage();
      }
    });
    prevPageButton.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        prevPage();
      }
    });
    okButton.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        close();
      }
    });
    addWindowListener(new WindowAdapter() {
      public void windowClosing(WindowEvent e) {
        close();
      }
      public void windowClosed(WindowEvent e) {
        close();
      }    
    });
  }

  private void nextPage() {
    ArrayList elements = getNextElements();
    if (elements != null)
      displayElements(elements);
  }
  
  private void prevPage() {
    ArrayList elements = getPreviousElements();
    if (elements != null)
      displayElements(elements);
  }
  
  private void displayElements(ArrayList elements) {
    EmployeeTableModel empTableModel = (EmployeeTableModel) employeeTable.getModel();
    if (empTableModel != null) {
      empTableModel.setData(elements);
      empTableModel.fireTableDataChanged();
    }
  }
  
  private ArrayList getNextElements() {
    ArrayList elements = null;
    try {
      elements = result.getNextElements(); // get next elements as page size is defined
    } catch(ApplicationException e) {
      handleException(e);
    }
    return elements;
  }
  
  private ArrayList getPreviousElements() {
    ArrayList elements = null;
    try {
      elements = result.getPreviousElements();
    } catch(ApplicationException ex) {
      System.out.println(ex.toString());
    }
    return elements;
  }

  private void openEmployeeDialog(Long empId) {
    try {
      EmployeeVO empVO =  empSession.findById(empId);
      new EmployeeDialog(this, empVO, transaction);
    } catch(ApplicationException e) {
      handleException(e);
    }
  }
  
  private void close() {
    try {
      result.close();
      transaction.end();
      jdbAccess.end();
      dispose();
    } catch (ApplicationException e) {
      handleException(e);
    }
  }
  
  public Image getImage(Class startClass, String fileName) {
    String packageName = startClass.getPackage().getName();
    String pathName = packageName.replace('.', '/');
    if (! pathName.equals(""))
      pathName = "/" + pathName; 
    String resourceName = pathName + "/" + fileName;
    URL url = startClass.getResource(resourceName);
    // if file does not exist in that package
    if (url == null)
      return null;
    String iconFileName = url.getFile();
    Image image = Toolkit.getDefaultToolkit().getImage(iconFileName);
    return image;
  }

  private void handleException(Exception e) {
    System.out.println(e.toString());
  }
  
  class EmployeeTableModel extends AbstractTableModel {
    private static final long serialVersionUID = 1L;
    private ArrayList fields;
    private ArrayList data;

    public EmployeeTableModel(ArrayList fields) {
      this.fields = fields;
    }
    
    private void setData(ArrayList data) {
      this.data = data;
    }
    
    private void setData(TransferObject transferObject, int row) {
      data.set(row, transferObject);
    }
    
    public int getColumnCount() {
      if (fields == null)
        return 0;
      else 
        return fields.size();
    }

    public int getRowCount() {
      if (data == null)
        return 0;
      return data.size();
    }

    public String getColumnName(int col) {
      Field f = (Field) fields.get(col);
      String colName = f.getName();
      return colName;
    }

    public Object getValueAt(int row, int col) {
      TransferObject to = getRow(row);
      String fieldName = getColumnName(col);
      Object value = null;
      try {
        value = to.getValue(fieldName);
      } catch (ApplicationException e) {
        handleException(e);
      }
      return value;
    }

    public Class getColumnClass(int col) {
      Object value = getValueAt(0, col);
      if (value == null)
        return Object.class;
      return value.getClass();
    }

    // all cells are not editable
    public boolean isCellEditable(int row, int col) {
      return false;
    }
    
    private TransferObject getRow(int position) {
      TransferObject to = null;
      if (data != null && position < data.size())
        to = (TransferObject) data.get(position);
      return to;
    }
  }
  
  class EmployeeMouseListener extends MouseAdapter {
    EmployeesFrame frame;
    JTable table;
    
    public EmployeeMouseListener(EmployeesFrame frame, JTable table) {
      this.frame = frame;
      this.table = table;
    }
    
    public void mouseClicked(MouseEvent event) {
      if (SwingUtilities.isLeftMouseButton(event) && event.getClickCount() >= 2) {
        Point p = event.getPoint();
        int row = table.rowAtPoint(p);
        Employee emp = (Employee)((EmployeeTableModel)table.getModel()).getRow(row);
        Long empId = emp.getId(); 
        openEmployeeDialog(empId);
      }
    }
  }
  
  class DateRenderer extends DefaultTableCellRenderer {
    private static final long serialVersionUID = 1L;
    DateFormat formatter;
    public DateRenderer() { 
      super();
    }

    public void setValue(Object value) {
      if (formatter==null) {
        formatter = DateFormat.getDateTimeInstance();
      }
      if (value instanceof Date) {
        setText((value == null) ? "" : formatter.format(value));
      }
    }
}

}
